/*!
 * \file main.c
 * \brief Pressure Click example
 *
 * # Description
 * This is a example which demonstrates the use of Pressure Click board.
 * Measured pressure and temperature data from the LPS331AP sensor on Pressure
 * click.
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * Initializes I2C/SPI serial interface and puts a device to the initial state.
 * Also initializes UART console module for results logging.
 *
 * ## Application Task
 * Reads the pressure and temperature results in standard units when
 * measurement was done and sends results to the console (usb uart terminal).
 * Repeats operation every 500ms.
 *
 * \author Nemanja Medakovic
 *
 */
// ------------------------------------------------------------------- INCLUDES

#include "board.h"
#include "log.h"
#include "pressure.h"


// ------------------------------------------------------------------ VARIABLES

static pressure_t pressure;
static log_t console;
static const uint8_t deg_cels[ 3 ] = { 176, 'C', 0 };

// ------------------------------------------------------ APPLICATION FUNCTIONS

void application_init( void )
{
    pressure_cfg_t pressure_cfg;
    log_cfg_t console_cfg;

    //  Click initialization.
    pressure_cfg_setup( &pressure_cfg );
    PRESSURE_MAP_MIKROBUS( pressure_cfg, MIKROBUS_1 );
    pressure_init( &pressure, &pressure_cfg );
    pressure_sw_reset( &pressure );
    pressure_default_cfg( &pressure );

    //  Console initialization.
    LOG_MAP_USB_UART( console_cfg );
    console_cfg.baud = 57600;
    log_init( &console, &console_cfg );
    log_write( &console, "***  Pressure initialization done  ***",
               LOG_FORMAT_LINE );
    log_write( &console, "**************************************",
               LOG_FORMAT_LINE );
}

void application_task( void )
{
    uint8_t status;
    float press;
    float temp;

    status = pressure_get_status( &pressure, PRESSURE_FLAG_MASK_P_DATA_RDY |
                                             PRESSURE_FLAG_MASK_T_DATA_RDY );

    while (!status)
    {
        status = pressure_get_status( &pressure, PRESSURE_FLAG_MASK_P_DATA_RDY |
                                                 PRESSURE_FLAG_MASK_T_DATA_RDY );
    }

    press = pressure_get_pressure( &pressure );
    temp = pressure_get_temperature( &pressure );

    log_printf( &console, "** Pressure is %.2f mbar\r\n", press );
    log_printf( &console, "** Temperature is %.2f ", temp );
    log_write( &console, deg_cels, LOG_FORMAT_LINE );
    log_write( &console, "**************************************",
               LOG_FORMAT_LINE );

    Delay_ms( 500 );
}

void main( void )
{
    application_init( );

    for ( ; ; )
    {
        application_task( );
    }
}


// ------------------------------------------------------------------------ END
